#include <stdlib.h>
#include <stdint.h>
#include "config.h"

#define HIGHCOLOR 255
#define LOWCOLOR 96

extern float displayScale;
extern uint8_t randomize;

/* Public Variables */
int8_t videoModified;

/* Hardware-like Registers */
uint8_t videoX;
uint8_t videoY;
uint8_t videoMode;
uint8_t videoA;

/* Private Variables */
static uint8_t videoValue;
static uint8_t videoColorValue;

static uint32_t videoColorIndex[16];
static uint8_t videoMemory[0x8000];
uint32_t renderMemory[(SDL_X_SIZE * SDL_Y_SIZE)];



void updatePalette(){
	uint8_t accentColors = videoMode & 0b11101110;
	for (uint8_t i = 0; i < 16; i++){
		uint32_t rawColor = 
			((i & 0b1) ? 
				HIGHCOLOR - (i & 0x8 ? 0 : LOWCOLOR) : 0) << 24 | 
			
			(((i & 0b10) ? 
				HIGHCOLOR - (i & 0x8 ? 0 : LOWCOLOR) : 0) << 16) | 
			
			(((i & 0b100) ? 
				HIGHCOLOR - (i & 0x8 ? 0 : LOWCOLOR) : 0) << 8) |
			
			0x000000FF;
		
		videoColorIndex[i] = rawColor;
	}
}

void initVideo(){
	videoModified = 1;
	videoMode = 0x01;
	updatePalette();
	
	if (randomize) {
		for (int i = 0; i < 0x8000; i++) videoMemory[i] = rand();
	}
}

void writeVideo(){
	videoMemory[videoX + (videoY * 128)] = videoA;
	videoModified = 1;
}

uint8_t readVideo(){
	return videoMemory[videoX + (videoY * 128)];
}

void setPixels(uint8_t x, uint8_t y){
	/* Check for Text or Graphics Mode */
	if ((videoMode & 0x01) == 0){
		/*  
		 *	Text Address Format
		 *  0bRRR1111AAAAAAAA
		 * 
		 *  R = row pins
		 *  A = address
		 */
		
		/* Get text index value */
		videoValue = videoMemory[(((y & 0xF8) >> 3) << 7) | x];
		
		/* Index the value in font location */
		videoValue = videoMemory[((y & 0x7) | 0b1111000) + (videoValue * 128)];
		videoColorValue = videoMemory[(((y & 0xF8) >> 3) << 7) + (x | 0x40)];
	}
	else {
		videoValue = videoMemory[(y * 128) + x];
		videoColorValue = videoMemory[(y * 128) + (x | 0x40)];
	}
	
	for (uint8_t i = 0; i < 8; i++){
		renderMemory[(y * SDL_X_SIZE) + ((x * 8) + i)] = 
				(videoValue & (128 >> i)) ? 
				videoColorIndex[((videoColorValue & 0xF0) >> 4)] : 
				videoColorIndex[(videoColorValue & 0x0F)];
	}
}

void updateVideo(){
	if (videoModified){		
		for (uint16_t y = 0; y < 256; y++){
			for (uint8_t x = 0; x < 48; x++){
				setPixels(x, (uint8_t)y);
			}
		}
		
		videoModified = 0;
	}
}
